#! /usr/bin/env python
# 

# ------------------------------
# 
# La grosse fleme de faire les comptes à chaque fois
# Maintenant c'est toi qui les fait pour moi :D
#
# ------------------------------


# ------------------------------
# Imports
# ------------------------------ 
import csv
import os
import optparse
from pathlib import Path

from account import Account

def extrait_from_file(file_name):
    """
    Extrait les informations à partir d'un fichier
    
    :param file_name: nom du fichier avec les comptes 
    :return: liste [[nom,montant,parts],...]
    """
    file = open(file_name, "r")
    try:
        reader = csv.reader(file, delimiter = ",")
        r_comptes =  list(reader)
        header = r_comptes.pop(0) # On récupère le premier élément (les headers)

        comptes = []

        for c in r_comptes: # On parcourt les lignes pour typer les éléments correctement
            compte = {}
            for (i,h) in enumerate(header):
                if "nom" in h.lower():
                    compte["nom"] = c[i]
                if "montant" in h.lower():
                    compte["montant"] = eval(c[i])
                if "part" in h.lower():
                    compte["parts"] = int(c[i])
            comptes += [compte]

        return comptes
    finally:
        file.close()

def forfait(compte, output = print):
    """
    Gère le nombre de parts et affiche quelques stats
    
    :param compte: comptes de ce que chacun a payé [[nom, montant,parts],...]
    :return: Comptes des crédits de chacun envers la communauté
    """
    cout_total = sum([c["montant"] for c in compte])
    output("Cout total: {cout_tot}".format(cout_tot = cout_total))

    nbr_jour = sum([c["parts"] for c in compte])
    output("Nombre de parts {njour}".format(njour = nbr_jour))

    cout_jour =  cout_total / nbr_jour
    output("Cout de la part: {cout}".format(cout = round(cout_jour,2)))

    output("\n")

    new_compte = []
    for pers in compte:
        new_compte += [[pers["nom"], (pers["montant"] - pers["parts"] * cout_jour)]]
        
    return new_compte

def normalise(compte):
    """Centre en 0 les comptes"""
    moyenne = sum([c["montant"] for c in compte]) / len(compte)
    compte_normalise = compte

    for (i,n) in enumerate(compte):
        compte_normalise[i]["montant"] = compte[i]["montant"] - moyenne

    return compte_normalise

def affiche_final(donRec, output = print):
    """
    Affiche qui donne quoi à qui à partir de la liste
    
    :param donRec: liste avec qui doit quoi à qui [[qui, àqui, quoi]...]
    """
    for g in donRec:
        output("{don} donne {montant} à {rec}".format(don = g[0], rec = g[1], montant = round(g[2],2)))

def join_comptes(compte1, compte2):
    """
    >>> c1 = [['pop', 3], ['bab', -2]]
    >>> c2 = [['pop', 1], ['nin', -3]]
    >>> join_comptes(c1, c2)
    [('nin', -3), ('pop', 4), ('bab', -2)]
    """
    c_dict1 = {k: v for k, v in compte1}
    c_dict2 = {k: v for k, v in compte2}
    
    joined = {k: c_dict1.get(k, 0) + c_dict2.get(k, 0) for k in set(c_dict1) | set(c_dict2)}
    return [(k, v) for k,v in joined.items()]

if __name__ == '__main__':
    # Pour analyser les options qu'on lui demande 
    parser = optparse.OptionParser()
    # options proposée
    parser.add_option("-f","--file",action="store", type = "string", dest="file_name", help="Analyse les comptes à partir du fichier donné en argument ou des fichiers du repertoire")
    #parser.add_option("-p","--path",action="store", type = "string", dest="pathname", help="Analyse les comptes à partir de tous les fichiers csv du repertoir (multicompte)")
    parser.add_option("-e","--seuil",action="store", type = "int", dest="seuil",default=0, help="Seuil à partir duquel on concidère qu'il n'est plus nécessaire de payer.")
    # Digestion
    (options, args) = parser.parse_args()

    if options.file_name:
        path = Path(options.file_name)
        assert path.exists()
        if path.is_dir():
            god = []
            for f in path.glob('*.csv'):
                print(f"Compte: {f.name}")
                compte = extrait_from_file(f)
                compte_normalise = forfait(compte)
                god = join_comptes(god, compte_normalise)

            account = Account(god)
        else:
            compte = extrait_from_file(options.file_name)
            # compte_normalise = normalise(compte)
            cred_dept = forfait(compte)
            #final = echange(compte_normalise, options.seuil)
            account = Account(cred_dept)

        final = account.tribut(options.seuil)
        affiche_final(final)


# -----------------------------
# Reglages pour 'vim'
# vim:set autoindent expandtab tabstop=4 shiftwidth=4:
# cursor: 16 del 

